@@ -2,8 +2,21 @@ import requests
 import warnings
 #import pandas as pd
 import json
+import xml.etree.ElementTree as ET
+from requests.models import Response
+
 warnings.filterwarnings("ignore")
 
+def get_response_object(status_code:int):
+    if status_code==409:
+        response = Response()
+        response.code = "conflict"
+        response.error_type = "conflict"
+        response.status_code = status_code
+        response._content = b'{ "status_code" : "409" ,"text":"name already exists"}'
+        return response
+
+
 
 class SplunkAlertAutomator():
     """This class can be used to build scripts to automate certain splunk tasks related to alerts and schedules searches.
@@ -225,6 +238,7 @@ class SplunkAlertAutomator():
 
 
 
+
 class SplunkDashboardAutomator():
     '''This class can be used to build scripts to automate certain splunk tasks related to dashboards.
 
@@ -245,29 +259,49 @@ class SplunkDashboardAutomator():
         self.password = password
         self.splunk_app = splunk_app
 
-    def __get_dashboard(self,dashboard_name:str):
+    def __delete_dashboard(self,dashboard_name:str):
+        
+        response = requests.delete(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/views/{dashboard_name}',
+                                verify=False,
+                               
+                                auth=(self.username, self.password))
+        return response
+
+
+    def delete_dashboard(self,dashboard_name:str):
+        return self.__delete_dashboard(dashboard_name=dashboard_name)
+
+
+    def delete_dashboard_list(self,dashboard_list:list):
+        response_list=[]
+        for dashboard_name in dashboard_list:
+            reponse_list.append(self.delete_dashboard(dashboard_name=dashboard_name))
+        return response_list
+
+    def __get_dashboard(self, dashboard_name: str):
         params = (
             ('output_mode', 'json'),
         )
         response = requests.get(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/views/{dashboard_name}',
-                                   verify=False,
-                                   params=params,                                  
-                                   auth=(self.username, self.password))
+                                verify=False,
+                                params=params,
+                                auth=(self.username, self.password))
         return response
 
-    def get_dashboard(self,dashboard_name:str):
-        response=self.__get_dashboard(
+    def get_dashboard(self, dashboard_name: str):
+        response = self.__get_dashboard(
             dashboard_name=dashboard_name
         )
-        payload=json.loads(response.content)
-        xml=payload['content']['eai:data']
-        acl=payload['acl']
+        content = json.loads(response.content)
+        xml=content['entry'][0]['content']['eai:data']
+        acl=content['entry'][0]['acl']
         return xml, acl
+        #return response
 
-    def get_dashboard_list(self,title_regex=["**"]):
+    def get_dashboard_list(self, title_regex=["**"]):
         '''returns a list object with alle the dashb'''
 
-        splunk_search=f"""
+        splunk_search = f"""
         | rest/servicesNS/-/{self.splunk_app}/data/ui/views splunk_server=local
 | table disabled title
         """
@@ -284,21 +318,119 @@ class SplunkDashboardAutomator():
                                  verify=False,
                                  auth=(self.username, self.password))
 
-        text=response.text
-        text_array=text.split('\n')
-        _list=[json.loads(text_e) for text_e in text_array if text_e is not '']
-        _list=[e['result']['title'] for e in _list] 
-        _list=list(dict.fromkeys(_list))#the fastest way to dedup a list https://stackoverflow.com/questions/7961363/removing-duplicates-in-lists/7961425#7961425
+        text = response.text
+        text_array = text.split('\n')
+        _list = [json.loads(text_e)
+                 for text_e in text_array if text_e is not '']
+        _list = [e['result']['title'] for e in _list]
+        # the fastest way to dedup a list https://stackoverflow.com/questions/7961363/removing-duplicates-in-lists/7961425#7961425
+        _list = list(dict.fromkeys(_list))
         return _list
 
+    def __create_dashboard(self, dashboard_name: str, xml: str):
 
-    def __update_dashboard(self, dashboard_name:str, xml:str):
+        params = (
+            ('output_mode', 'json'),
+        )
+        dashboard_config = {
+            'name': dashboard_name,
+            'eai:data': xml
+        }
+        response = requests.post(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/views',
+                                 data=dashboard_config,
+                                 verify=False,
+                                 params=params,
+                                 auth=(self.username, self.password))
+
+        return response
+
+    def create_dashboard(self, dashboard_name: str, xml: str, title:str=None, overwrite=False):
+
+        exists_response=self.__get_dashboard(dashboard_name=dashboard_name)
+        if exists_response.status_code==200:
+            print(f"{dashboard_name} already exists.")
+            
+            if overwrite==False:
+                return get_response_object(status_code=409)
+            else:
+                del_response=self.__delete_dashboard(dashboard_name=dashboard_name)
+                print(f"{del_response.status_code} Deleted existing dashboard {dashboard_name}")
+                if del_response.status_code!=200:
+                    print("delete failed. check dashboard acl, permissions")
+                    return get_response_object(status_code=409)
+
+
+
+        if title:
+            tree = ET.ElementTree(ET.fromstring(xml))
+            root=tree.getroot()
+            root.find('label').text=title
+            xml=ET.tostring(root)
+
+        response=self.__create_dashboard(dashboard_name,xml)
+        return response
+
+    def __update_dashboard(self, dashboard_name: str, xml: str):
         pass
 
-    def update_dashboard(self, dashboard_name:str, xml:str):
+    def update_dashboard(self, dashboard_name: str, xml: str):
         pass
 
 
+    def update_navbar(self,xml:str):
+        #experimental does not work yet. default.xml cannot be found for some reason
+        navbar_config = {
+            
+            'eai:data': xml
+        }
+        response = requests.post(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/nav/default',
+                                 data=navbar_config,
+                                 verify=False,
+                                
+                                 auth=(self.username, self.password))
+
+        return response
+
+
+    def get_navbar(self):
+        params = (
+            ('output_mode', 'json'),
+        )
+        
+        response = requests.get(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/nav/default',
+                                 params=params,
+                                 verify=False,
+                                
+                                 auth=(self.username, self.password))
+
+        content = json.loads(response.content)
+        xml=content['entry'][0]['content']['eai:data']
+        acl=content['entry'][0]['acl']
+        return xml, acl
+
+
+    def get_dashboard_acl(self, dashboard_name):
+        params = (
+            ('output_mode', 'json'),
+        )
+        response = requests.get(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/views/{dashboard_name}/acl',
+                                 
+                                 verify=False,
+                                 params=params,
+                                 auth=(self.username, self.password))
+
+        return json.loads(response.content)['entry'][0]['acl']
+
+    def change_dashboard_acl(self,dashboard_name, dashboard_acl_dict):
+
+        
+        response = requests.post(f'{self.splunk_uri}:8089/servicesNS/{self.username}/{self.splunk_app}/data/ui/views/{dashboard_name}/acl',
+                                 verify=False,
+                                 data=dashboard_acl_dict,
+                                 auth=(self.username, self.password))
+        print(f"{response.status_code}, while changing ACL of {dashboard_name}")                         
+        return response
+
 
 class SplunkAutomator():
     """This class can be used to build scripts to automate certain splunk tasks.
@@ -367,8 +499,26 @@ class SplunkMigrater():
 
         return response_list_move,response_list_acl
 
-    def move_dashboard(self):
-        pass
+    def move_dashboard(self, dashboard_name:str, overwrite:bool=False, dashboard_acl_dict:dict=None):
+        '''moves an dash from from_splunk to to_splunk'''
+        xml, _=self.from_splunk.dashboards.get_dashboard(dashboard_name=dashboard_name)
+        
+        r_create=self.to_splunk.dashboards.create_dashboard(dashboard_name=dashboard_name, xml=xml, overwrite=overwrite)
+        #create_dashboard(self, dashboard_name: str, xml: str, title:str=None, overwrite=False):
+        r_acl=None
+        if dashboard_acl_dict and (r_create.status_code==201):
+            r_acl=self.to_splunk.dashboards.change_dashboard_acl(dashboard_name=dashboard_name, dashboard_acl_dict= dashboard_acl_dict)
 
-    def move_dashboard_list(self):
-        pass
\ No newline at end of file
+
+        return r_create,r_acl
+
+    def move_dashboard_list(self, dashboard_list:list, overwrite:bool=False,dashboard_acl_dict:dict=None):
+        '''moves all dashboards in the list from from_splunk to to_splunk'''
+        response_list_move=[]
+        response_list_acl=[]
+        for dashboard_name in dashboard_list:
+            r_move,r_acl=self.move_dashboard( dashboard_name=dashboard_name, overwrite=overwrite,dashboard_acl_dict= dashboard_acl_dict)
+            response_list_move.append(r_move)
+            response_list_acl.append(r_acl)
+
+        return response_list_move,response_list_acl
\ No newline at end of file
